import QtQuick 2.0
import Sailfish.Silica 1.0

Page {
    id: mainPage

    // The effective value will be restricted by ApplicationWindow.allowedOrientations
    allowedOrientations: Orientation.All

    Component {
        id: roomContextMenu
        ContextMenu {
            MenuItem {
                text: "Remove"
                onClicked: highlight()
            }
        }
    }

    SilicaListView {
        id: roomView
        anchors.fill: parent

        ScrollDecorator { flickable: roomView }

        // PullDownMenu and PushUpMenu must be declared in SilicaFlickable, SilicaListView or SilicaGridView
        PullDownMenu {
            MenuItem {
                text: qsTr("About")
                onClicked: pageStack.push(Qt.resolvedUrl("AboutPage.qml"))
            }
            MenuItem {
                text: qsTr("Logout")
                onClicked: {
                    login.logout()
                    pageStack.replace(Qt.resolvedUrl("LoginPage.qml"))
                }
            }
        }

        header: PageHeader {
            title: qsTr("Rooms")
        }

        model: rooms
        signal sectionClicked(string name)

        delegate: ListItem {
            id: roomItem
            menu: roomContextMenu

            onClicked: pageStack.push(Qt.resolvedUrl("ChatPage.qml"), {room: model.roomModel} );

            property bool collapsed: false
            visible: !collapsed

            contentHeight: collapsed ? 0 : (mainRow.height + Theme.paddingSmall)
            contentWidth: parent.width

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: Theme.paddingMedium
                width: roomItem.width

                id: mainRow

                Label {
                    BackgroundItem {
                        Rectangle {
                            color: Theme.highlightBackgroundColor
                        }
                    }

                    color: unreadNotifications > 0 ? Theme.highlightColor : Theme.secondaryColor
                    width: 2 * Theme.fontSizeLarge
                    id: unreadMessages
                    text: unreadNotifications
                    font.pixelSize: unreadNotifications < 100 ? Theme.fontSizeHuge : Theme.fontSizeLarge
                    horizontalAlignment: Text.AlignRight
                }

                Column {
                    width: mainRow.width - unreadMessages.width
                    - roomImage.width - 3* Theme.paddingMedium
                    Label {
                        id: roomName
                        text: model.roomName
                        color: Theme.primaryColor
                        font.pixelSize: Theme.fontSizeLarge
                    }

                    Text {
                        anchors {
                            left: parent.left
                            right: parent.right
                        }

                        id: lastMessage
                        text: model.lastMessage
                        color: unreadNotifications > 0 ? Theme.primaryColor : Theme.secondaryColor
                        wrapMode: Text.Wrap
                        elide: Text.ElideRight
                        maximumLineCount: 3
                    }
                }

                Image {
                    id: roomImage
                    width: unreadMessages.width
                    height: unreadMessages.height
                    sourceSize.width: unreadMessages.width
                    sourceSize.height: unreadMessages.height
                    fillMode: Image.PreserveAspectFit
                    source: model.roomPicture
                    asynchronous: true
                }
            }

            Connections {
                target: roomItem.ListView.view
                onSectionClicked: if (roomItem.ListView.section === name)
                collapsed = !collapsed
            }
        }


        section.property: "tag"
        section.criteria: ViewSection.FullString
        section.delegate: ExpandingSection {
            title: section
            onExpandedChanged: {
                roomView.sectionClicked(section)
            }
        }
    }
}
