//#ifdef QT_QML_DEBUG
#include <QtQuick>
//#endif
#include <QMetaType>

#include <sailfishapp.h>

#include "login.h"
#include "mxcdownloader.h"
#include "mxcimageprovider.h"
#include "settings.h"
#include "sync.h"

#include "models/roomlistmodel.h"

Q_DECLARE_METATYPE(std::vector<std::string>);

QDataStream &operator<<(QDataStream &out, const std::string &s) {
    out << QString::fromStdString(s);
    return out;
}
QDataStream &operator>>(QDataStream &in, std::string &s) {
    QString qs;
    in >> qs;
    s = qs.toStdString();
    return in;
}

int main(int argc, char *argv[]) {
    // SailfishApp::main() will display "qml/harbour-spoon.qml", if you need more
    // control over initialization, you can use:
    //
    //   - SailfishApp::application(int, char *[]) to get the QGuiApplication *
    //   - SailfishApp::createView() to get a new QQuickView * instance
    //   - SailfishApp::pathTo(QString) to get a QUrl to a resource file
    //   - SailfishApp::pathToMainQml() to get a QUrl to the main QML file
    //

    // return SailfishApp::main(argc, argv);

    // To display the view, call "show()" (will show fullscreen on device).
    // Set up QML engine.
    // qRegisterMetaType<Room>("Room");
    qmlRegisterUncreatableType<EventType>("dev.neko.spoon", 1, 0, "EventType", "Can't instantiate EventType");
    qRegisterMetaType<std::vector<mtx::events::collections::TimelineEvents>>(
        "std::vector<mtx::events::collections::TimelineEvents>");
    qRegisterMetaType<std::vector<std::string>>("std::vector<std::string>");
    qRegisterMetaType<QSharedPointer<Room>>("QSharedPointer<Room>");
    qRegisterMetaType<std::string>("std::string");
    qRegisterMetaTypeStreamOperators<std::string>("std::string");

    QScopedPointer<QGuiApplication> app(SailfishApp::application(argc, argv));
    QScopedPointer<QQuickView> v(SailfishApp::createView());

    v->engine()->addImageProvider("MXCImage", new MXCImageProvider());

    Login login;
    v->rootContext()->setContextProperty("login", &login);

    MXCDownloader downloader;
    v->rootContext()->setContextProperty("mxcDownloader", &downloader);

    RoomListModel rooms;
    Sync sync(rooms, &login);
    QObject::connect(&login, &Login::loginSuccess, &sync, &Sync::startSync, Qt::QueuedConnection);
    QObject::connect(&sync, &Sync::newRoom, &rooms, &RoomListModel::addRoom, Qt::QueuedConnection);
    QObject::connect(&sync, &Sync::newInvite, &rooms, &RoomListModel::addInvite, Qt::QueuedConnection);
    QObject::connect(&sync, &Sync::roomUpdated, &rooms, &RoomListModel::updateRoom, Qt::QueuedConnection);
    QObject::connect(app.data(), &QGuiApplication::aboutToQuit, &sync, &Sync::exitSync, Qt::QueuedConnection);
    RoomListSortProxy roomsSorted;
    roomsSorted.setSourceModel(&rooms);
    roomsSorted.setDynamicSortFilter(true);
    roomsSorted.sort(0);

    v->rootContext()->setContextProperty("rooms", &roomsSorted);

    // If you wish to publish your app on the Jolla harbour, it is recommended
    // that you prefix your internal namespaces with "harbour.".
    //
    // For details see:
    // https://harbour.jolla.com/faq#1.5.0
    // qmlRegisterType<DemoModel>("com.example", 1, 0, "DemoModel");

    // Start the application.
    v->setSource(SailfishApp::pathTo("qml/harbour-spoon.qml"));
    v->show();

    return app->exec();
}
