#include "mxcdownloader.h"

#include <QDataStream>
#include <QDebug>
#include <QDir>
#include <QFile>
#include <QStandardPaths>

#include "client.h"
#include "debug_out.h"

MXCDownloadResponse::MXCDownloadResponse(QString url) {
        QFileInfo cacheFile(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/" +
                            QString(url).remove("mxc://"));

        if (cacheFile.exists()) {
            this->m_location = cacheFile.filePath();
            qDebug() << "File already cached: " << url;
            return;
        }

        qDebug() << "Downloading file: " << url;

        http::client().download(url.toStdString(),
                                [this, cacheFile, url](const std::string &res, const std::string &content_type,
                                                       const std::string &originalFilename, mtx::http::RequestErr err) {
                                    if (err) {
                                        qDebug() << "Failed to download file: " << url << " error: " << *err;
                                        emit failed(QString::fromStdString(err->matrix_error.error));
                                        return;
                                    }

                                    // m_image.setText("original filename", QString::fromStdString(originalFilename));
                                    // m_image.setText("mxc url", "mxc://" + m_id);

                                    QDir().mkpath(cacheFile.path());
                                    QFile file(cacheFile.filePath());
                                    if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate))
                                        emit failed("Failed to open cache file");
                                    if (QDataStream(&file).writeRawData(res.data(), res.size()) == -1)
                                        emit failed("Failed to write cache file");

                                    this->m_location = cacheFile.filePath();
                                    emit downloaded(m_location);
                                });
}

bool MXCDownloader::cached(QString url) {
    QFileInfo cacheFile(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/" +
                        QString(url).remove("mxc://"));

    return cacheFile.exists();
}

